#include "Wallpapers.h"
#include <algorithm>
#include <filesystem>
#include <iostream>
#include <ostream>
#include <stdexcept>

Wallpapers::Wallpapers(const std::string& directoryPath) : directoryPath(directoryPath) {

	for (const auto& entry : std::filesystem::directory_iterator(directoryPath)) {
		if (!isValidPath(entry.path())) {
			continue;
		}
		wallpapersPath.push_back(entry.path());
	}
	if (wallpapersPath.size() == 0) {
		throw std::runtime_error("Directory '" + directoryPath + "' doesn't contain any valid wallapers\n\
Allowed extension are .png, .jpg, .jpeg");
	}
	e1 = std::default_random_engine(r());
	uniformDist = std::uniform_int_distribution<int>(0, wallpapersPath.size());
	wallpapersPathQueue = wallpapersPath;
}

void Wallpapers::shuffle() {
	wallpapersPathQueue = wallpapersPath;
	std::ranges::shuffle(wallpapersPathQueue, e1);
}

const std::string& Wallpapers::next() {
	if (wallpapersPathQueue.size() == 0) {
		throw std::runtime_error("No valid wallpapers found when calling Wallpapers::next method");
	}
	const std::string& wallpaperPath = wallpapersPathQueue[currentWallpaperIndex];
	currentWallpaperIndex = (currentWallpaperIndex + 1) % wallpapersPathQueue.size();
	return wallpaperPath;
}

bool Wallpapers::isValidPath(const std::string& path) const {
	static std::array validExtensions {"jpg", "jpeg", "png"};

	for (const auto& extension : validExtensions) {
		if (path.ends_with(extension)) {
			return true;
		}
	}
	return false;
}
